import luigi
from servicecatalog_puppet import manifest_utils

from servicecatalog_puppet.workflow import tasks


class ManifestTask(tasks.PuppetTask):
    manifest_file_path = luigi.Parameter()
    puppet_account_id = luigi.Parameter()

    def params_for_results_display(self):
        return {
            "puppet_account_id": self.puppet_account_id,
            "manifest_file_path": self.manifest_file_path,
        }

    def run(self):
        self.info("started")
        with open(self.manifest_file_path, "r") as m:
            manifest = manifest_utils.load(m, self.puppet_account_id)
        self.write_output(manifest)
        self.info("Finished")


class SectionTask(tasks.PuppetTask):
    manifest_file_path = luigi.Parameter()
    puppet_account_id = luigi.Parameter()
    should_use_sns = luigi.BoolParameter()
    should_use_product_plans = luigi.BoolParameter()
    include_expanded_from = luigi.BoolParameter()
    single_account = luigi.Parameter()
    is_dry_run = luigi.BoolParameter()
    execution_mode = luigi.Parameter()

    def params_for_results_display(self):
        return {
            "puppet_account_id": self.puppet_account_id,
            "manifest_file_path": self.manifest_file_path,
        }

    def requires(self):
        return {
            "manifest": ManifestTask(
                manifest_file_path=self.manifest_file_path,
                puppet_account_id=self.puppet_account_id,
            ),
        }

    @property
    def manifest(self):
        return manifest_utils.Manifest(self.load_from_input("manifest"))



import unittest


class DeleteCloudFormationStackTaskComponentTest(unittest.TestCase):
    account_id = "0123456789010"
    region = "eu-west-0"
    stack_name = "delete-me"

    def setUp(self) -> None:
        from . import general

        self.sut = general.DeleteCloudFormationStackTask(
            self.account_id, self.region, self.stack_name
        )

    def test_params_for_results_display(self):
        self.assertEqual(
            {
                "account_id": self.account_id,
                "region": self.region,
                "stack_name": self.stack_name,
            },
            self.sut.params_for_results_display(),
        )

    def test_api_calls_used(self):
        self.assertEqual(
            {
                f"cloudformation.describe_stacks_single_page_{self.account_id}_{self.region}": 1,
                f"cloudformation.delete_stack_{self.account_id}_{self.region}": 1,
                f"cloudformation.describe_stack_events_{self.account_id}_{self.region}": 1,
            },
            self.sut.api_calls_used(),
        )



import unittest

import pytest


class GeneratePoliciesTemplateTest(unittest.TestCase):
    region = "eu-west-0"
    puppet_account_id = "01234567890"
    manifest_file_path = "tcvyuiho"

    def setUp(self) -> None:
        from . import generate

        self.sut = generate.GeneratePoliciesTemplate(
            region=self.region,
            puppet_account_id=self.puppet_account_id,
            manifest_file_path=self.manifest_file_path,
            should_use_sns=True,
            should_use_product_plans=True,
            include_expanded_from=True,
            single_account=True,
            is_dry_run=True,
            execution_mode="hub",
            sharing_policies=dict(),
        )

    def test_params_for_results_display(self):
        expected_result = {
            "region": self.region,
            "puppet_account_id": self.puppet_account_id,
            "manifest_file_path": self.manifest_file_path,
        }
        self.assertEqual(expected_result, self.sut.params_for_results_display())


class EnsureEventBridgeEventBusTaskTest(unittest.TestCase):
    region = "eu-west-0"
    puppet_account_id = "01234567890"

    def setUp(self) -> None:
        from . import generate

        self.sut = generate.EnsureEventBridgeEventBusTask(
            region=self.region, puppet_account_id=self.puppet_account_id,
        )

    def test_params_for_results_display(self):
        expected_result = {
            "region": self.region,
            "puppet_account_id": self.puppet_account_id,
        }
        self.assertEqual(expected_result, self.sut.params_for_results_display())


class GeneratePoliciesTest(unittest.TestCase):
    region = "eu-west-0"
    puppet_account_id = "01234567890"
    manifest_file_path = "tcvyuiho"
    sharing_policies = dict()

    should_use_sns = False
    should_use_product_plans = True
    include_expanded_from = True
    single_account = None
    is_dry_run = False
    execution_mode = "hub"

    def setUp(self) -> None:
        from . import generate

        self.sut = generate.GeneratePolicies(
            region=self.region,
            puppet_account_id=self.puppet_account_id,
            manifest_file_path=self.manifest_file_path,
            sharing_policies=self.sharing_policies,
            should_use_sns=self.should_use_sns,
            should_use_product_plans=self.should_use_product_plans,
            include_expanded_from=self.include_expanded_from,
            single_account=self.single_account,
            is_dry_run=self.is_dry_run,
            execution_mode=self.execution_mode,
        )

    def test_params_for_results_display(self):
        expected_result = {
            "region": self.region,
            "puppet_account_id": self.puppet_account_id,
            "manifest_file_path": self.manifest_file_path,
        }
        self.assertEqual(expected_result, self.sut.params_for_results_display())

    def test_requires(self):
        from . import generate

        expected_result = {
            "template": generate.GeneratePoliciesTemplate(
                manifest_file_path=self.manifest_file_path,
                puppet_account_id=self.puppet_account_id,
                region=self.region,
                should_use_sns=self.should_use_sns,
                should_use_product_plans=self.should_use_product_plans,
                include_expanded_from=self.include_expanded_from,
                single_account=self.single_account,
                is_dry_run=self.is_dry_run,
                execution_mode=self.execution_mode,
                sharing_policies=self.sharing_policies,
            ),
        }
        self.assertEqual(expected_result, self.sut.requires())


class GenerateSharesTaskTest(unittest.TestCase):
    puppet_account_id = "01234567890"
    manifest_file_path = "tcvyuiho"

    should_use_sns = False
    should_use_product_plans = True
    include_expanded_from = True
    single_account = None
    is_dry_run = False
    execution_mode = "hub"

    def setUp(self) -> None:
        from . import generate

        self.sut = generate.GenerateSharesTask(
            puppet_account_id=self.puppet_account_id,
            manifest_file_path=self.manifest_file_path,
            should_use_sns=self.should_use_sns,
            should_use_product_plans=self.should_use_product_plans,
            include_expanded_from=self.include_expanded_from,
            single_account=self.single_account,
            is_dry_run=self.is_dry_run,
            execution_mode=self.execution_mode,
        )

    def test_params_for_results_display(self):
        expected_result = {
            "puppet_account_id": self.puppet_account_id,
            "manifest_file_path": self.manifest_file_path,
        }
        self.assertEqual(expected_result, self.sut.params_for_results_display())



import luigi

from servicecatalog_puppet import config, constants

from servicecatalog_puppet.workflow import manifest as manifest_tasks
from servicecatalog_puppet.workflow import (
    portfoliomanagement as portfoliomanagement_tasks,
)
from servicecatalog_puppet.workflow import general as general_tasks
from servicecatalog_puppet.workflow import tasks
from betterboto import client as betterboto_client


class GeneratePoliciesTemplate(manifest_tasks.SectionTask):
    region = luigi.Parameter()
    sharing_policies = luigi.DictParameter()

    def output(self):
        return luigi.LocalTarget(f"output/{self.uid}.template.yaml")

    def params_for_results_display(self):
        return {
            "region": self.region,
            "puppet_account_id": self.puppet_account_id,
            "manifest_file_path": self.manifest_file_path,
        }

    def run(self):
        rendered = config.env.get_template("policies.template.yaml.j2").render(
            sharing_policies=self.sharing_policies,
            VERSION=config.get_puppet_version(),
            HOME_REGION=self.region,
        )
        with self.output().open("w") as output_file:
            output_file.write(rendered)


class EnsureEventBridgeEventBusTask(tasks.PuppetTask):
    puppet_account_id = luigi.Parameter()
    region = luigi.Parameter()

    def params_for_results_display(self):
        return {
            "region": self.region,
            "puppet_account_id": self.puppet_account_id,
        }

    def run(self):
        with betterboto_client.CrossAccountClientContextManager(
            "events",
            f"arn:aws:iam::{self.puppet_account_id}:role/servicecatalog-puppet/PuppetRole",
            f"events-{self.puppet_account_id}-{self.region}",
            region_name=self.region,
        ) as events:
            created = False
            try:
                events.describe_event_bus(Name=constants.EVENT_BUS_NAME)
                created = True
            except events.exceptions.ResourceNotFoundException:
                events.create_event_bus(Name=constants.EVENT_BUS_NAME)
        self.write_output(created)


class GeneratePolicies(manifest_tasks.SectionTask):
    region = luigi.Parameter()
    sharing_policies = luigi.DictParameter()

    def params_for_results_display(self):
        return {
            "region": self.region,
            "puppet_account_id": self.puppet_account_id,
            "manifest_file_path": self.manifest_file_path,
        }

    def requires(self):
        return {
            "template": GeneratePoliciesTemplate(
                manifest_file_path=self.manifest_file_path,
                puppet_account_id=self.puppet_account_id,
                should_use_sns=self.should_use_sns,
                should_use_product_plans=self.should_use_product_plans,
                include_expanded_from=self.include_expanded_from,
                single_account=self.single_account,
                is_dry_run=self.is_dry_run,
                execution_mode=self.execution_mode,
                region=self.region,
                sharing_policies=self.sharing_policies,
            ),
        }

    def run(self):
        self.info("running")
        template = self.read_from_input("template")
        with betterboto_client.CrossAccountClientContextManager(
            "cloudformation",
            f"arn:aws:iam::{self.puppet_account_id}:role/servicecatalog-puppet/PuppetRole",
            f"cf-{self.puppet_account_id}-{self.region}",
            region_name=self.region,
        ) as cloudformation:
            cloudformation.create_or_update(
                StackName="servicecatalog-puppet-policies",
                TemplateBody=template,
                NotificationARNs=[
                    f"arn:aws:sns:{self.region}:{self.puppet_account_id}:servicecatalog-puppet-cloudformation-regional-events"
                ]
                if self.should_use_sns
                else [],
            )
        self.write_output(template)


class GenerateSharesTask(manifest_tasks.SectionTask):
    def params_for_results_display(self):
        return {
            "puppet_account_id": self.puppet_account_id,
            "manifest_file_path": self.manifest_file_path,
        }

    def run(self):
        self.info("running")
        for region_name, accounts in self.manifest.get_accounts_by_region().items():
            yield general_tasks.DeleteCloudFormationStackTask(
                account_id=self.puppet_account_id,
                region=region_name,
                stack_name="servicecatalog-puppet-shares",
            )

        for (
            region_name,
            sharing_policies,
        ) in self.manifest.get_sharing_policies_by_region().items():
            yield EnsureEventBridgeEventBusTask(
                puppet_account_id=self.puppet_account_id, region=region_name,
            )

            yield GeneratePolicies(
                manifest_file_path=self.manifest_file_path,
                puppet_account_id=self.puppet_account_id,
                should_use_sns=self.should_use_sns,
                should_use_product_plans=self.should_use_product_plans,
                include_expanded_from=self.include_expanded_from,
                single_account=self.single_account,
                is_dry_run=self.is_dry_run,
                execution_mode=self.execution_mode,
                region=region_name,
                sharing_policies=sharing_policies,
            )

        self.info("Finished generating policies")
        for (
            region_name,
            shares_by_portfolio_account,
        ) in self.manifest.get_shares_by_region_portfolio_account(
            self.puppet_account_id
        ).items():
            for (
                portfolio_name,
                shares_by_account,
            ) in shares_by_portfolio_account.items():
                for account_id, share in shares_by_account.items():
                    yield portfoliomanagement_tasks.CreateShareForAccountLaunchRegion(
                        manifest_file_path=self.manifest_file_path,
                        puppet_account_id=self.puppet_account_id,
                        account_id=account_id,
                        region=region_name,
                        portfolio=portfolio_name,
                    )

        self.write_output(self.params_for_results_display())



import functools
import os

import pkg_resources
import yaml
from betterboto import client as betterboto_client
from jinja2 import Environment, FileSystemLoader

from servicecatalog_puppet import asset_helpers
from servicecatalog_puppet import constants

import logging

logger = logging.getLogger()
logger.setLevel(logging.INFO)


@functools.lru_cache(maxsize=32)
def get_config(puppet_account_id, default_region=None):
    if os.path.exists("config.yaml"):
        with open("config.yaml", "r") as f:
            conf = yaml.safe_load(f.read())
            return conf

    logger.info("getting config,  default_region: {}".format(default_region))
    region = default_region if default_region else get_home_region(puppet_account_id)
    with betterboto_client.CrossAccountClientContextManager(
        "ssm",
        f"arn:aws:iam::{puppet_account_id}:role/servicecatalog-puppet/PuppetRole",
        f"{puppet_account_id}-{region}-PuppetRole",
        region_name=region,
    ) as ssm:
        response = ssm.get_parameter(Name=constants.CONFIG_PARAM_NAME)
        return yaml.safe_load(response.get("Parameter").get("Value"))


@functools.lru_cache(maxsize=32)
def get_regions(puppet_account_id, default_region=None):
    logger.info("getting regions,  default_region: {}".format(default_region))
    return get_config(puppet_account_id, default_region).get("regions")


@functools.lru_cache(maxsize=32)
def get_should_use_sns(puppet_account_id, default_region=None):
    logger.info("getting should_use_sns,  default_region: {}".format(default_region))
    return get_config(puppet_account_id, default_region).get(
        "should_collect_cloudformation_events", True
    )


@functools.lru_cache(maxsize=32)
def get_should_use_eventbridge(puppet_account_id, default_region=None):
    logger.info(
        "getting should_use_eventbridge,  default_region: {}".format(default_region)
    )
    return get_config(puppet_account_id, default_region).get(
        "should_forward_events_to_eventbridge", False
    )


@functools.lru_cache(maxsize=32)
def get_should_forward_failures_to_opscenter(puppet_account_id, default_region=None):
    logger.info(
        "getting should_forward_failures_to_opscenter,  default_region: {}".format(
            default_region
        )
    )
    return get_config(puppet_account_id, default_region).get(
        "should_forward_failures_to_opscenter", False
    )


@functools.lru_cache(maxsize=32)
def get_should_use_product_plans(puppet_account_id, default_region=None):
    logger.info(
        "getting should_use_product_plans,  default_region: {}".format(default_region)
    )
    return get_config(puppet_account_id, default_region).get(
        "should_use_product_plans", True
    )


@functools.lru_cache(maxsize=32)
def get_local_config(what):
    if os.path.exists("config.yaml"):
        with open("config.yaml", "r") as f:
            conf = yaml.safe_load(f.read())
            return conf.get(what, None)
    else:
        return None


@functools.lru_cache()
def get_home_region(puppet_account_id):
    if get_local_config("home_region"):
        return get_local_config("home_region")
    with betterboto_client.CrossAccountClientContextManager(
        "ssm",
        f"arn:aws:iam::{puppet_account_id}:role/servicecatalog-puppet/PuppetRole",
        f"{puppet_account_id}-PuppetRole",
    ) as ssm:
        response = ssm.get_parameter(Name=constants.HOME_REGION_PARAM_NAME)
        return response.get("Parameter").get("Value")


@functools.lru_cache(maxsize=32)
def get_org_iam_role_arn(puppet_account_id):
    with betterboto_client.CrossAccountClientContextManager(
        "ssm",
        f"arn:aws:iam::{puppet_account_id}:role/servicecatalog-puppet/PuppetRole",
        f"{puppet_account_id}-PuppetRole",
        region_name=get_home_region(puppet_account_id),
    ) as ssm:
        try:
            response = ssm.get_parameter(
                Name=constants.CONFIG_PARAM_NAME_ORG_IAM_ROLE_ARN
            )
            return response.get("Parameter").get("Value")
        except ssm.exceptions.ParameterNotFound:
            logger.info("No org role set")
            return None


template_dir = asset_helpers.resolve_from_site_packages("templates")
env = Environment(loader=FileSystemLoader(template_dir), extensions=["jinja2.ext.do"],)


@functools.lru_cache(maxsize=32)
def get_puppet_account_id():
    with betterboto_client.ClientContextManager("sts") as sts:
        return sts.get_caller_identity().get("Account")


@functools.lru_cache(maxsize=32)
def get_current_account_id():
    with betterboto_client.ClientContextManager("sts") as sts:
        return sts.get_caller_identity().get("Account")


# TODO - not used?
def get_ssm_config_for_parameter(account_ssm_param, required_parameter_name):
    if account_ssm_param.get("region") is not None:
        return {
            "name": account_ssm_param.get("name"),
            "region": account_ssm_param.get("region"),
            "parameter_name": required_parameter_name,
        }
    else:
        return {
            "name": account_ssm_param.get("name"),
            "parameter_name": required_parameter_name,
        }


def get_puppet_version():
    return pkg_resources.get_distribution("aws-service-catalog-puppet").version
